function [n y] = BoyleLauBarrierTree(Spot,K,oldBar,T,r,v,old_n,PutCall,EuroAmer,BarType,M1,M2)

% Returns the price of a single barrier option using a binomial tree.
% Finds the optimal number of steps in the tree using the method
% of Boyle and Tau.
% Spot = Spot price
% K = Strike price
% oldBar = Barrier without monitoring adjustment
% T = Maturity
% r = Risk free rate
% v = volatility
% old_n = Original number of steps
% PutCall = 'P'ut or 'C'all.
% EuroAmer = 'E'uropean or 'A'merican.
% BarType = 'DI', 'DO', 'UI', 'UO'
% M1 = Barrier monitoring: # of observations.
% M2 = Barrier monitoring
%      'C'ontinuous, 'H'ourly, 'D'aily,
%      'W'eekly, 'M'onthly

% Barrier Monitoring.  Creates a new barrier.
Bar = NewBarrier(Spot,oldBar,T,v,M1,M2);

% Barrier Conditions
if (strcmp(BarType, 'DO') | strcmp(BarType, 'DI')) & Bar > Spot
    disp('Error: Barrier Must be Below Spot Price for a Down-and-Out or Down-and-In option')
	n = 0; y = 0;
	return;
elseif (strcmp(BarType, 'UO') | strcmp(BarType, 'UI')) & Bar < Spot
    disp('Error: Barrier Must be Above Spot Price for a Up-and-Out or Up-and-In option')
	n = 0; y = 0;
	return;
else
	for m = 1:100
		F(m) = m^2*v^2*T / (log(Spot/Bar))^2;
	end
	if old_n < F(1)
		disp(['Increase the number steps to at least ' num2str(floor(F(1)+1))])
		n = 0; y = 0;
		return;
	else
		for i = 1:99
			if (F(i) < old_n) & (old_n < F(i+1))
				n = floor(F(i+1));
			end
		end
	end
end

% CRR Tree Parameters
dt = T/n;
u = exp(v*sqrt(dt));
d = 1/u;
p = (exp(r*dt)-d)/(u-d);
exp_rT = exp(-r*dt);

% Initialize option prices.
V = zeros(n+1,n+1);

% Terminal option prices.
for i = 1:n+1
    AssetPrice = Spot*u^(n+1-i)*d^(i-1);
    if ((strcmp(BarType, 'DO') | strcmp(BarType, 'DI')) & AssetPrice <= Bar) | ...
       ((strcmp(BarType, 'UO') | strcmp(BarType, 'UI')) & AssetPrice >= Bar)
        V(i,n+1) = 0;
    elseif strcmp(PutCall, 'C')
        V(i,n+1) = max(AssetPrice - K,0);
    elseif strcmp(PutCall, 'P')
        V(i,n+1) = max(K - AssetPrice,0);
	end
end

% Remaining option prices./
for j = n:-1:1
	for i = 1:j
		AssetPrice = Spot*u^(j-i)*d^(i-1);
		if ((strcmp(BarType, 'DO') | strcmp(BarType, 'DI')) & AssetPrice <= Bar) | ...
		   ((strcmp(BarType, 'UO') | strcmp(BarType, 'UI')) & AssetPrice >= Bar)
	       V(i,j) = 0;
		else
			switch EuroAmer
				case 'E'
					V(i,j) = exp_rT*(p*V(i,j+1) + (1-p)*V(i+1,j+1));
				case 'A'
					if strcmp(PutCall, 'C')
						V(i,j) = max(AssetPrice - K, exp_rT*(p*V(i,j+1) + (1-p)*V(i+1,j+1)));
					elseif strcmp(PutCall, 'P')
						V(i,j) = max(K - AssetPrice, exp_rT*(p*V(i,j+1) + (1-p)*V(i+1,j+1)));
					end
			end
		end
	end
end

% Price DO and UO Euro and American calls and puts directly
% Price DI and UI Euro calls and puts from in-out barrier parity relationship
% Price DI Amer call and UI Amer puts from reflection principle.
% DI Amer puts and UI Amer call prices not available.

if strcmp(BarType, 'DO') | strcmp(BarType, 'UO')
    y = V(1,1);
else
	switch EuroAmer
		case 'E'
			if strcmp(PutCall, 'C')
				y = BinomialTree(Spot,K,r,v,T,n,'C','E') - V(1,1);
			elseif strcmp(PutCall, 'P')
				y = BinomialTree(Spot,K,r,v,T,n,'P','E') - V(1,1);
			end
		case 'A'
			if strcmp(PutCall, 'C') & strcmp(BarType, 'DI')
				y = (Spot/Bar)^(1-2*r/v^2)*BinomialTree(Bar^2/Spot,K,r,v,T,n,'C','A');
			elseif strcmp(PutCall, 'P') & strcmp(BarType, 'UI')
				y = (Spot/Bar)^(1-2*r/v^2)*BinomialTree(Bar^2/Spot,K,r,v,T,n,'P','A');
			elseif strcmp(PutCall, 'P') & strcmp(BarType, 'DI');
				disp('You Cannot Price an American Down-and-In Put Using This Algorithm')
 				n = 0; y = 0;
				return;
			elseif (strcmp(PutCall, 'C') & strcmp(BarType, 'UI'))
				disp('You Cannot Price an American Up-and-In Call Using This Algorithm')
				n = 0; y = 0;
				return;
			end
	end
end

